@@ -8,21 +8,19 @@
 #include <stdint.h>
 #include <stdio.h>
 
+#ifdef ENABLE_PCAP_L7_MUTATOR
+#include "pl7m.h"
+#endif
+
 struct ndpi_workflow_prefs *prefs = NULL;
 struct ndpi_workflow *workflow = NULL;
+struct ndpi_global_context *g_ctx;
 
-int nDPI_LogLevel = 0;
-char *_debug_protocols = NULL;
-u_int32_t current_ndpi_memory = 0, max_ndpi_memory = 0;
-u_int8_t enable_protocol_guess = 1, enable_payload_analyzer = 0;
+u_int8_t enable_payload_analyzer = 0;
 u_int8_t enable_flow_stats = 1;
 u_int8_t human_readeable_string_len = 5;
-u_int8_t max_num_udp_dissected_pkts = 16 /* 8 is enough for most protocols, Signal requires more */, max_num_tcp_dissected_pkts = 80 /* due to telnet */;
-ndpi_init_prefs init_prefs = ndpi_track_flow_payload | ndpi_enable_ja3_plus | ndpi_enable_tcp_ack_payload_heuristic;
-int enable_malloc_bins = 1;
+u_int8_t max_num_udp_dissected_pkts = 0, max_num_tcp_dissected_pkts = 0; /* Disable limits at application layer */;
 int malloc_size_stats = 0;
-int max_malloc_bins = 14;
-struct ndpi_bin malloc_bins; /* unused */
 
 extern void ndpi_report_payload_stats(FILE *out);
 
@@ -30,6 +28,13 @@ extern void ndpi_report_payload_stats(FILE *out);
 extern int force_no_aesni;
 #endif
 
+#ifdef ENABLE_PCAP_L7_MUTATOR
+size_t LLVMFuzzerCustomMutator(uint8_t *Data, size_t Size,
+                               size_t MaxSize, unsigned int Seed) {
+  return pl7m_mutator(Data, Size, MaxSize, Seed);
+}
+#endif
+
 int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
   pcap_t * pkts;
   const u_char *pkt;
@@ -55,28 +60,35 @@ int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
     fuzz_set_alloc_callbacks();
 #endif
 
-    workflow = ndpi_workflow_init(prefs, NULL /* pcap handler will be set later */, 0, ndpi_serialization_format_json);
-    // enable all protocols
-    NDPI_BITMASK_SET_ALL(all);
-    ndpi_set_protocol_detection_bitmask2(workflow->ndpi_struct, &all);
+    g_ctx = ndpi_global_init();
 
+    workflow = ndpi_workflow_init(prefs, NULL /* pcap handler will be set later */, 0, ndpi_serialization_format_json, g_ctx);
+
+    ndpi_workflow_set_flow_callback(workflow, NULL, NULL); /* No real callback */
+
+    ndpi_set_config(workflow->ndpi_struct, NULL, "log.level", "3");
+    ndpi_set_config(workflow->ndpi_struct, "all", "log", "1");
+
+    ndpi_load_domain_suffixes(workflow->ndpi_struct, "public_suffix_list.dat");
+    ndpi_load_categories_dir(workflow->ndpi_struct, "./lists/");
     ndpi_load_protocols_file(workflow->ndpi_struct, "protos.txt");
     ndpi_load_categories_file(workflow->ndpi_struct, "categories.txt", NULL);
     ndpi_load_risk_domain_file(workflow->ndpi_struct, "risky_domains.txt");
     ndpi_load_malicious_ja3_file(workflow->ndpi_struct, "ja3_fingerprints.csv");
     ndpi_load_malicious_sha1_file(workflow->ndpi_struct, "sha1_fingerprints.csv");
 
-    ndpi_set_detection_preferences(workflow->ndpi_struct, ndpi_pref_enable_tls_block_dissection, 0 /* unused */);
+    // enable all protocols
+    NDPI_BITMASK_SET_ALL(all);
+    ndpi_set_protocol_detection_bitmask2(workflow->ndpi_struct, &all);
 
-    ndpi_set_monitoring_state(workflow->ndpi_struct, NDPI_PROTOCOL_STUN,
-                              10, NDPI_MONITORING_STUN_SUBCLASSIFIED);
+    ndpi_set_config(workflow->ndpi_struct, NULL, "packets_limit_per_flow", "255");
+    ndpi_set_config(workflow->ndpi_struct, NULL, "flow.track_payload", "1");
+    ndpi_set_config(workflow->ndpi_struct, NULL, "tcp_ack_payload_heuristic", "1");
+    ndpi_set_config(workflow->ndpi_struct, "tls", "application_blocks_tracking", "1");
+    ndpi_set_config(workflow->ndpi_struct, "stun", "max_packets_extra_dissection", "255");
+    ndpi_set_config(workflow->ndpi_struct, "zoom", "max_packets_extra_dissection", "255");
+    ndpi_set_config(workflow->ndpi_struct, "rtp", "search_for_stun", "1");
 
-    memset(workflow->stats.protocol_counter, 0,
-	   sizeof(workflow->stats.protocol_counter));
-    memset(workflow->stats.protocol_counter_bytes, 0,
-	   sizeof(workflow->stats.protocol_counter_bytes));
-    memset(workflow->stats.protocol_flows, 0,
-	   sizeof(workflow->stats.protocol_flows));
     ndpi_finalize_initialization(workflow->ndpi_struct);
 
 #ifdef CRYPT_FORCE_NO_AESNI
@@ -125,9 +137,10 @@ int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
 
     if(packet_checked) {
       ndpi_risk flow_risk;
+      struct ndpi_flow_info *flow = NULL; /* unused */
 
       memcpy(packet_checked, pkt, header->caplen);
-      ndpi_workflow_process_packet(workflow, header, packet_checked, &flow_risk);
+      ndpi_workflow_process_packet(workflow, header, packet_checked, &flow_risk, &flow);
       free(packet_checked);
     }
 
@@ -139,7 +152,7 @@ int LLVMFuzzerTestOneInput(const uint8_t *Data, size_t Size) {
   for(i = 0; i < workflow->prefs.num_roots; i++)
     ndpi_tdestroy(workflow->ndpi_flows_root[i], ndpi_flow_info_freer);
   ndpi_free(workflow->ndpi_flows_root);
-  /* Free payload analyzer data, without printing */
+  /* Free payload analyzer data */
   if(enable_payload_analyzer)
     ndpi_report_payload_stats(stdout);
 
