@@ -23,6 +23,7 @@
 #define NDPI_CURRENT_PROTO NDPI_PROTOCOL_ETHERNET_IP
 
 #include "ndpi_api.h"
+#include "ndpi_private.h"
 
 static void ndpi_int_ethernet_ip_add_connection(struct ndpi_detection_module_struct
 						*ndpi_struct, struct ndpi_flow_struct *flow) {
@@ -31,8 +32,8 @@ static void ndpi_int_ethernet_ip_add_connection(struct ndpi_detection_module_str
 }
 
 
-void ndpi_search_ethernet_ip(struct ndpi_detection_module_struct *ndpi_struct,
-			     struct ndpi_flow_struct *flow) {
+static void ndpi_search_ethernet_ip(struct ndpi_detection_module_struct *ndpi_struct,
+				    struct ndpi_flow_struct *flow) {
   struct ndpi_packet_struct *packet = &ndpi_struct->packet;
   
   NDPI_LOG_DBG(ndpi_struct, "search for ETHERNET_IP\n");
@@ -60,8 +61,8 @@ void ndpi_search_ethernet_ip(struct ndpi_detection_module_struct *ndpi_struct,
   
 
 void init_ethernet_ip_dissector(struct ndpi_detection_module_struct *ndpi_struct,
-				u_int32_t *id, NDPI_PROTOCOL_BITMASK *detection_bitmask) {
-  ndpi_set_bitmask_protocol_detection("EthernetIP", ndpi_struct, detection_bitmask, *id,
+				u_int32_t *id) {
+  ndpi_set_bitmask_protocol_detection("EthernetIP", ndpi_struct, *id,
 				      NDPI_PROTOCOL_ETHERNET_IP,
 				      ndpi_search_ethernet_ip,
 				      NDPI_SELECTION_BITMASK_PROTOCOL_V4_V6_TCP_WITH_PAYLOAD_WITHOUT_RETRANSMISSION,
