@@ -27,13 +27,15 @@
 #define NDPI_CURRENT_PROTO NDPI_PROTOCOL_SIP
 
 #include "ndpi_api.h"
+#include "ndpi_private.h"
 
 static void ndpi_int_sip_add_connection(struct ndpi_detection_module_struct *ndpi_struct,
-					struct ndpi_flow_struct *flow,
-					u_int8_t due_to_correlation) {
+					struct ndpi_flow_struct *flow) {
   ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_SIP, NDPI_PROTOCOL_UNKNOWN, NDPI_CONFIDENCE_DPI);
 }
 
+/* ********************************************************** */
+
 #if !defined(WIN32)
 static inline
 #elif defined(MINGW_GCC)
@@ -41,13 +43,16 @@ __mingw_forceinline static
 #else
 __forceinline static
 #endif
-void ndpi_search_sip_handshake(struct ndpi_detection_module_struct
-			       *ndpi_struct, struct ndpi_flow_struct *flow)
-{
+void ndpi_search_sip(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow) {
   struct ndpi_packet_struct *packet = &ndpi_struct->packet;
   const u_int8_t *packet_payload = packet->payload;
   u_int32_t payload_len = packet->payload_packet_len;
 
+  if(flow->packet_counter >= 8) {
+    NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
+    return;
+  }
+  
   if(payload_len > 4) {
     /* search for STUN Turn ChannelData Prefix */
     u_int16_t message_len = ntohs(get_u_int16_t(packet->payload, 2));
@@ -57,14 +62,25 @@ void ndpi_search_sip_handshake(struct ndpi_detection_module_struct
       payload_len -= 4;
       packet_payload += 4;
     }
+
+    if(!isprint(packet_payload[0])) {
+      NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
+      return;
+    }
   }
 
-  if(payload_len >= 14) {
+  if(payload_len == 5 && memcmp(packet_payload, "hello", 5) == 0) {
+    NDPI_LOG_INFO(ndpi_struct, "found sip via HELLO (kind of ping)\n");
+    ndpi_int_sip_add_connection(ndpi_struct, flow);
+    return;
+  }
+
+  if(payload_len >= 14) {    
     if((memcmp(packet_payload, "NOTIFY ", 7) == 0 || memcmp(packet_payload, "notify ", 7) == 0)
        && (memcmp(&packet_payload[7], "SIP:", 4) == 0 || memcmp(&packet_payload[7], "sip:", 4) == 0)) {
 
       NDPI_LOG_INFO(ndpi_struct, "found sip NOTIFY\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
 
@@ -72,14 +88,14 @@ void ndpi_search_sip_handshake(struct ndpi_detection_module_struct
        && (memcmp(&packet_payload[9], "SIP:", 4) == 0 || memcmp(&packet_payload[9], "sip:", 4) == 0)) {
 
       NDPI_LOG_INFO(ndpi_struct, "found sip REGISTER\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
 
     if((memcmp(packet_payload, "INVITE ", 7) == 0 || memcmp(packet_payload, "invite ", 7) == 0)
        && (memcmp(&packet_payload[7], "SIP:", 4) == 0 || memcmp(&packet_payload[7], "sip:", 4) == 0)) {
       NDPI_LOG_INFO(ndpi_struct, "found sip INVITE\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
 
@@ -91,20 +107,20 @@ void ndpi_search_sip_handshake(struct ndpi_detection_module_struct
     /*
       if(memcmp(packet_payload, "SIP/2.0 200 OK", 14) == 0 || memcmp(packet_payload, "sip/2.0 200 OK", 14) == 0) {
       NDPI_LOG_INFO(ndpi_struct, "found sip SIP/2.0 0K\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
       }
     */
     if(memcmp(packet_payload, "SIP/2.0 ", 8) == 0 || memcmp(packet_payload, "sip/2.0 ", 8) == 0) {
       NDPI_LOG_INFO(ndpi_struct, "found sip SIP/2.0 *\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
 
     if((memcmp(packet_payload, "BYE ", 4) == 0 || memcmp(packet_payload, "bye ", 4) == 0)
        && (memcmp(&packet_payload[4], "SIP:", 4) == 0 || memcmp(&packet_payload[4], "sip:", 4) == 0)) {
       NDPI_LOG_INFO(ndpi_struct, "found sip BYE\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
 
@@ -112,7 +128,7 @@ void ndpi_search_sip_handshake(struct ndpi_detection_module_struct
        && ((memcmp(&packet_payload[4], "SIP:", 4) == 0 || memcmp(&packet_payload[4], "sip:", 4) == 0) ||
            (memcmp(&packet_payload[4], "TEL:", 4) == 0 || memcmp(&packet_payload[4], "tel:", 4) == 0))) {
       NDPI_LOG_INFO(ndpi_struct, "found sip ACK\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
 
@@ -120,21 +136,21 @@ void ndpi_search_sip_handshake(struct ndpi_detection_module_struct
        && ((memcmp(&packet_payload[7], "SIP:", 4) == 0 || memcmp(&packet_payload[7], "sip:", 4) == 0) ||
            (memcmp(&packet_payload[7], "TEL:", 4) == 0 || memcmp(&packet_payload[7], "tel:", 4) == 0))) {
       NDPI_LOG_INFO(ndpi_struct, "found sip CANCEL\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
 
     if((memcmp(packet_payload, "PUBLISH ", 8) == 0 || memcmp(packet_payload, "publish ", 8) == 0)
        && (memcmp(&packet_payload[8], "SIP:", 4) == 0 || memcmp(&packet_payload[8], "sip:", 4) == 0)) {
       NDPI_LOG_INFO(ndpi_struct, "found sip PUBLISH\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
 
     if((memcmp(packet_payload, "SUBSCRIBE ", 10) == 0 || memcmp(packet_payload, "subscribe ", 10) == 0)
        && (memcmp(&packet_payload[10], "SIP:", 4) == 0 || memcmp(&packet_payload[10], "sip:", 4) == 0)) {
       NDPI_LOG_INFO(ndpi_struct, "found sip SUBSCRIBE\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
         
@@ -142,7 +158,7 @@ void ndpi_search_sip_handshake(struct ndpi_detection_module_struct
     if((memcmp(packet_payload, "MESSAGE ", 8) == 0 || memcmp(packet_payload, "message ", 8) == 0)
        && (memcmp(&packet_payload[8], "SIP:", 4) == 0 || memcmp(&packet_payload[8], "sip:", 4) == 0)) {
       NDPI_LOG_INFO(ndpi_struct, "found sip MESSAGE\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
 
@@ -152,56 +168,48 @@ void ndpi_search_sip_handshake(struct ndpi_detection_module_struct
        && ((memcmp(&packet_payload[8], "SIP:", 4) == 0 || memcmp(&packet_payload[8], "sip:", 4) == 0) ||
            (memcmp(&packet_payload[8], "TEL:", 4) == 0 || memcmp(&packet_payload[8], "tel:", 4) == 0))) {
       NDPI_LOG_INFO(ndpi_struct, "found sip OPTIONS\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
 
     if((memcmp(packet_payload, "REFER ", 6) == 0 || memcmp(packet_payload, "refer ", 6) == 0)
        && (memcmp(&packet_payload[6], "SIP:", 4) == 0 || memcmp(&packet_payload[6], "sip:", 4) == 0)) {
       NDPI_LOG_INFO(ndpi_struct, "found sip REFER\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
 
     if((memcmp(packet_payload, "PRACK ", 6) == 0 || memcmp(packet_payload, "prack ", 6) == 0)
        && (memcmp(&packet_payload[6], "SIP:", 4) == 0 || memcmp(&packet_payload[6], "sip:", 4) == 0)) {
       NDPI_LOG_INFO(ndpi_struct, "found sip PRACK\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
 
     if((memcmp(packet_payload, "INFO ", 5) == 0 || memcmp(packet_payload, "info ", 5) == 0)
        && (memcmp(&packet_payload[5], "SIP:", 4) == 0 || memcmp(&packet_payload[5], "sip:", 4) == 0)) {
       NDPI_LOG_INFO(ndpi_struct, "found sip INFO\n");
-      ndpi_int_sip_add_connection(ndpi_struct, flow, 0);
+      ndpi_int_sip_add_connection(ndpi_struct, flow);
       return;
     }
   }
 
   /* add bitmask for tcp only, some stupid udp programs
    * send a very few (< 10 ) packets before invite (mostly a 0x0a0x0d, but just search the first 3 payload_packets here */
-  if(packet->udp != NULL && flow->packet_counter < 10) {
+  if(packet->udp != NULL) {
     NDPI_LOG_DBG2(ndpi_struct, "need next packet\n");
     return;
   }
 
   if(payload_len == 4 && get_u_int32_t(packet_payload, 0) == 0) {
-    NDPI_LOG_DBG2(ndpi_struct, "maybe sip. need next packet\n");
+    NDPI_LOG_DBG2(ndpi_struct, "maybe sip. need next packet\n");    
     return;
   }
-
-  NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
 }
 
-static void ndpi_search_sip(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow)
-{
-  NDPI_LOG_DBG(ndpi_struct, "search sip\n");
+/* ********************************************************** */
 
-  ndpi_search_sip_handshake(ndpi_struct, flow);
-}
-
-void init_sip_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id)
-{
+void init_sip_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id) {
   ndpi_set_bitmask_protocol_detection("SIP", ndpi_struct, *id,
 				      NDPI_PROTOCOL_SIP,
 				      ndpi_search_sip,
