@@ -1,5 +1,4 @@
 #!/usr/bin/env bash
-
 set -e
 
 FAILED_ASN=0
@@ -7,15 +6,18 @@ TOTAL_ASN=0
 
 function processing_list() {
 	local LIST_MERGED="/tmp/list_m"
+	local LIST6_MERGED="/tmp/list6_m"
 
 	echo "(2) Processing IP addresses..."
 	./mergeipaddrlist.py "$1" > $LIST_MERGED
-	./ipaddr2list.py "$LIST_MERGED" "$2" > "$3"
-	rm -f $LIST_MERGED
+	./mergeipaddrlist.py "$2" > $LIST6_MERGED
+	./ipaddr2list.py "$LIST_MERGED" "$3" "$LIST6_MERGED" > "$4"
+	rm -f $LIST_MERGED $LIST6_MERGED
 }
 
 function create_list() {
 	LIST=/tmp/list
+	LIST6=/tmp/list6
 
 	for i in "${@:4}"; do
 		TOTAL_ASN=$(( TOTAL_ASN + 1 ))
@@ -23,15 +25,20 @@ function create_list() {
 			echo "Could not fetch route for ${i} (${1})"
 			FAILED_ASN=$(( FAILED_ASN + 1 ))
 		fi
+		if ! ./get_routes6_by_asn.sh "$i" >> $LIST6; then
+			echo "Could not fetch route6 for ${i} (${1})"
+			FAILED_ASN=$(( FAILED_ASN + 1 ))
+		fi
 	done
 
+	#TODO: ipv6 addresses
 	if [ ! -z "$3" ];  then
 	    # Split comma separated list of additional networks to add
 	    echo "$3" | tr "," "\n" >> $LIST
 	fi
 
-	processing_list "$LIST" "$1" "$2"
-	rm -f $LIST
+	processing_list "$LIST" "$LIST6" "$1" "$2"
+	rm -f $LIST $LIST6
 }
 
 cd "$(dirname "${0}")" || exit 1
@@ -122,11 +129,6 @@ DEST=../src/lib/inc_generated/ndpi_asn_bloomberg.c.inc
 create_list NDPI_PROTOCOL_BLOOMBERG $DEST "" "AS8188" "AS58850" "AS33220" "AS33181" "AS199559" "AS17063" "AS13908" "AS10361"
 echo "(3) Bloomberg IPs are available in $DEST"
 
-echo "(1) Downloading Citrix routes..."
-DEST=../src/lib/inc_generated/ndpi_asn_citrix.c.inc
-create_list NDPI_PROTOCOL_CITRIX $DEST "" "AS395424" "AS21866" "AS213380" "AS20104" "AS16815"
-echo "(3) Citrix IPs are available in $DEST"
-
 echo "(1) Downloading Edgecast routes..."
 DEST=../src/lib/inc_generated/ndpi_asn_edgecast.c.inc
 create_list NDPI_PROTOCOL_EDGECAST $DEST "" "AS15133"
@@ -207,7 +209,7 @@ DEST=../src/lib/inc_generated/ndpi_asn_roblox.c.inc
 create_list NDPI_PROTOCOL_ROBLOX $DEST "" "AS22697"
 echo "(3) Roblox IPs are available in $DEST"
 
-if [ ${TOTAL_ASN} -eq 0 -o ${TOTAL_ASN} -eq ${FAILED_ASN} ]; then
+if [ ${TOTAL_ASN} -eq 0 ] || [ ${TOTAL_ASN} -eq ${FAILED_ASN} ]; then
 	printf '%s: %s\n' "${0}" "All download(s) failed, ./get_routes_by_asn.sh broken?"
 	exit 1
 else
