@@ -1,4 +1,4 @@
-#/bin/bash
+#!/usr/bin/env bash
 
 cd "$(dirname "${0}")" || exit 1
 
@@ -36,10 +36,14 @@ RETVAL=$(( RETVAL + $? ))
 
 ./gambling_sites_download.sh
 RETVAL=$(( RETVAL + $? ))
+./malicious_sites_download.sh
+RETVAL=$(( RETVAL + $? ))
 ./mullvad_ip_addresses_download.sh
 RETVAL=$(( RETVAL + $? ))
 ./protonvpn_ip_addresses_download.sh
 RETVAL=$(( RETVAL + $? ))
+./public_suffix_list_download.sh
+RETVAL=$(( RETVAL + $? ))
 
 test ${RETVAL} -ne 0 && printf '%s: %s\n' "${0}" "${RETVAL} script(s) failed"
 exit ${RETVAL}
